@@ -24,7 +24,6 @@
 #endif
 
 #include <gst/gst.h>
-#include <gst/interfaces/mixer.h>
 
 #include <libxfce4util/libxfce4util.h>
 #include <libxfce4ui/libxfce4ui.h>
@@ -74,7 +73,7 @@ struct _XfceMixerContainer
 
   GstElement  *card;
 
-  GHashTable  *widgets;
+  GList       *widgets;
 
   gulong       message_handler_id;
 };
@@ -111,7 +110,7 @@ static void
 xfce_mixer_container_init (XfceMixerContainer *mixer_container)
 {
   mixer_container->card = NULL;
-  mixer_container->widgets = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
+  mixer_container->widgets = NULL;
   mixer_container->message_handler_id = 0;
 }
 
@@ -140,7 +139,7 @@ xfce_mixer_container_finalize (GObject *object)
   xfce_mixer_bus_disconnect (mixer_container->message_handler_id);
 
   g_object_unref (mixer_container->card);
-  g_hash_table_unref (mixer_container->widgets);
+  /*g_hash_table_unref (mixer_container->widgets);*/
 
   (*G_OBJECT_CLASS (xfce_mixer_container_parent_class)->finalize) (object);
 }
@@ -214,18 +213,19 @@ xfce_mixer_container_create_contents (XfceMixerContainer *mixer_container)
   XfceMixerPreferences *preferences;
   XfceMixerTrackType    type;
   GstMixerTrack        *track;
+  GstMixerTrack        *parent;
   const GList          *iter;
+  const GList          *iter1;
   const gchar          *titles[4] = { N_("_Playback"), N_("C_apture"), N_("S_witches"), N_("_Options") };
-  GtkWidget            *label_alignment;
-  GtkWidget            *option_alignment;
   GtkWidget            *track_widget;
   GtkWidget            *track_label_widget;
   GtkWidget            *labels[4];
   GtkWidget            *scrollwins[4];
   GtkWidget            *views[4];
   GtkWidget            *last_separator[4] = { NULL, NULL, NULL, NULL };
-  GtkWidget            *alignment;
   GtkWidget            *vbox;
+  GtkWidget            *hbox;
+  GtkWidget            *card_track;
   GtkWidget            *label1;
   GtkWidget            *label2;
   GtkWidget            *label3;
@@ -247,17 +247,10 @@ xfce_mixer_container_create_contents (XfceMixerContainer *mixer_container)
       gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrollwins[i]), GTK_SHADOW_IN);
       gtk_container_set_border_width (GTK_CONTAINER (scrollwins[i]), 6);
       gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrollwins[i]), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-
-      if (i < 2)
-        views[i] = gtk_table_new (1, 2, FALSE);
-      else
-        views[i] = gtk_table_new (2, 1, FALSE);
-
-      gtk_table_set_col_spacings (GTK_TABLE (views[i]), 12);
-      gtk_table_set_row_spacings (GTK_TABLE (views[i]), 6);
-
-      gtk_container_set_border_width (GTK_CONTAINER (views[i]), 6);
-      gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (scrollwins[i]), views[i]);
+      gtk_scrolled_window_set_overlay_scrolling(GTK_SCROLLED_WINDOW (scrollwins[i]), FALSE);
+      views[i] = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
+      g_object_set (G_OBJECT (views[i]), "border-width", 6, NULL);
+      gtk_container_add (GTK_CONTAINER (scrollwins[i]), views[i]);
       gtk_viewport_set_shadow_type (GTK_VIEWPORT (gtk_bin_get_child (GTK_BIN (scrollwins[i]))), GTK_SHADOW_NONE);
       gtk_widget_show (views[i]);
       gtk_widget_show (scrollwins[i]);
@@ -270,99 +263,185 @@ xfce_mixer_container_create_contents (XfceMixerContainer *mixer_container)
         {
           track = GST_MIXER_TRACK (iter->data);
 
-          track_label = xfce_mixer_get_track_label (track);
-
+          track_label = gst_mixer_track_get_name (track);
+#if 0
           if (!xfce_mixer_preferences_get_control_visible (preferences, track_label))
             continue;
-
+#endif
           /* Determine the type of the mixer track */
           type = xfce_mixer_track_type_new (track);
 
-          switch (type) 
+          switch (type)
             {
             case XFCE_MIXER_TRACK_TYPE_PLAYBACK:
+              hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 6);
               /* Create a regular volume control for this track */
               track_label_widget = gtk_label_new (track_label);
-              gtk_table_attach (GTK_TABLE (views[0]), track_label_widget,
-                                num_children[0], num_children[0] + 1, 0, 1, GTK_SHRINK, GTK_SHRINK, 0, 0);
+              gtk_box_pack_start(GTK_BOX(hbox), track_label_widget, FALSE, FALSE, 1);
               gtk_widget_show (track_label_widget);
+              g_object_set (G_OBJECT (track_label_widget), "halign", GTK_ALIGN_START, NULL);
+
+              gtk_widget_show (hbox);
+              gtk_box_pack_start(GTK_BOX(views[0]), hbox, FALSE, FALSE, 1);
               track_widget = xfce_mixer_track_new (mixer_container->card, track);
-              gtk_table_attach (GTK_TABLE (views[0]), track_widget,
-                                num_children[0], num_children[0] + 1, 1, 2, GTK_SHRINK, GTK_FILL|GTK_EXPAND, 0, 0);
+
+              if (GST_MIXER_TRACK_HAS_FLAG(track, GST_MIXER_TRACK_SOFTWARE))
+              {
+                int pos = 0;
+                card_track = gtk_combo_box_text_new();
+                gtk_box_pack_start(GTK_BOX(hbox), card_track, FALSE, FALSE, 1);
+
+                for (iter1 = gst_mixer_list_tracks (GST_MIXER (mixer_container->card)); iter1 != NULL; iter1 = g_list_next (iter1))
+                {
+                  parent = GST_MIXER_TRACK (iter1->data);
+                  if (GST_MIXER_TRACK_HAS_FLAG(parent, GST_MIXER_TRACK_OUTPUT) &&
+                      !GST_MIXER_TRACK_HAS_FLAG(parent, GST_MIXER_TRACK_SOFTWARE))
+                  {
+                    gchar *tid;
+
+                    tid = g_strdup_printf("%d", gst_mixer_track_get_id (parent));
+
+                    gtk_combo_box_text_insert(GTK_COMBO_BOX_TEXT(card_track),
+                                              pos,
+                                              tid,
+                                              gst_mixer_track_get_name(parent));
+                    g_free(tid);
+
+                    if (gst_mixer_track_get_parent_track_id(track) ==
+                        gst_mixer_track_get_id (parent))
+                    {
+                      gtk_combo_box_set_active(GTK_COMBO_BOX(card_track),
+                                               pos);
+                    }
+                    pos++;
+                  }
+                }
+                /* Only show if we have more than 2 outputs */
+                if (pos > 1)
+                {
+                  gtk_widget_show(card_track);
+                  /* Connect to changed signal */
+                  xfce_mixer_track_connect (XFCE_MIXER_TRACK(track_widget), card_track);
+                }
+              }
+
+              num_children[0]++;
+              gtk_box_pack_start(GTK_BOX(views[0]), track_widget, FALSE, TRUE, 1);
+              g_object_set (G_OBJECT (track_widget), "halign", GTK_ALIGN_FILL, "vexpand", FALSE, NULL);
               gtk_widget_show (track_widget);
               num_children[0]++;
 
               /* Append a separator. The last one will be destroyed later */
-              last_separator[0] = gtk_vseparator_new ();
-              gtk_table_attach (GTK_TABLE (views[0]), last_separator[0], 
-                                num_children[0], num_children[0] + 1, 0, 2, GTK_SHRINK, GTK_FILL|GTK_EXPAND, 0, 0);
+              last_separator[0] = gtk_separator_new (GTK_ORIENTATION_HORIZONTAL);
+              gtk_box_pack_start(GTK_BOX(views[0]), last_separator[0], FALSE, FALSE, 1);
               gtk_widget_show (last_separator[0]);
               num_children[0]++;
 
               /* Add the track to the hash table */
-              g_hash_table_insert (mixer_container->widgets, g_strdup (track_label), track_widget);
+              g_object_set_data (G_OBJECT(track), "track-widget", track_widget);
+              mixer_container->widgets = g_list_append (mixer_container->widgets, track_widget);
               break;
 
             case XFCE_MIXER_TRACK_TYPE_CAPTURE:
+              hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 6);
               /* Create a regular volume control for this track */
               track_label_widget = gtk_label_new (track_label);
-              gtk_table_attach (GTK_TABLE (views[1]), track_label_widget,
-                                num_children[1], num_children[1] + 1, 0, 1, GTK_SHRINK, GTK_SHRINK, 0, 0);
+              gtk_box_pack_start(GTK_BOX(hbox), track_label_widget, FALSE, FALSE, 1);
               gtk_widget_show (track_label_widget);
+              g_object_set (G_OBJECT (track_label_widget), "halign", GTK_ALIGN_START, NULL);
+
+              gtk_widget_show (hbox);
+              gtk_box_pack_start(GTK_BOX(views[1]), hbox, FALSE, FALSE, 1);
               track_widget = xfce_mixer_track_new (mixer_container->card, track);
-              gtk_table_attach (GTK_TABLE (views[1]), track_widget,
-                                num_children[1], num_children[1] + 1, 1, 2, GTK_SHRINK, GTK_FILL|GTK_EXPAND, 0, 0);
+
+              if (GST_MIXER_TRACK_HAS_FLAG(track, GST_MIXER_TRACK_SOFTWARE))
+              {
+                int pos = 0;
+                card_track = gtk_combo_box_text_new();
+                gtk_box_pack_start(GTK_BOX(hbox), card_track, FALSE, FALSE, 1);
+
+                for (iter1 = gst_mixer_list_tracks (GST_MIXER (mixer_container->card)); iter1 != NULL; iter1 = g_list_next (iter1))
+                {
+                  parent = GST_MIXER_TRACK (iter1->data);
+                  if (GST_MIXER_TRACK_HAS_FLAG(parent, GST_MIXER_TRACK_INPUT) &&
+                      !GST_MIXER_TRACK_HAS_FLAG(parent, GST_MIXER_TRACK_SOFTWARE))
+                  {
+                    gchar *tid;
+
+                    tid = g_strdup_printf("%d", gst_mixer_track_get_id (parent));
+
+                    gtk_combo_box_text_insert(GTK_COMBO_BOX_TEXT(card_track),
+                                              pos,
+                                              tid,
+                                              gst_mixer_track_get_name(parent));
+                    g_free(tid);
+
+                    if (gst_mixer_track_get_parent_track_id(track) ==
+                        gst_mixer_track_get_id (parent))
+                    {
+                      gtk_combo_box_set_active(GTK_COMBO_BOX(card_track),
+                                               pos);
+                    }
+                    pos++;
+                  }
+                }
+                /* Only show if we have more than 2 inputs */
+                if (pos > 1)
+                {
+                  gtk_widget_show(card_track);
+                  /* Connect to changed signal */
+                  xfce_mixer_track_connect (XFCE_MIXER_TRACK(track_widget), card_track);
+                }
+              }
+
+              num_children[1]++;
+              g_object_set (G_OBJECT (track_widget), "halign", GTK_ALIGN_FILL, "vexpand", FALSE, NULL);
+              gtk_box_pack_start(GTK_BOX(views[1]), track_widget, FALSE, TRUE, 1);
               gtk_widget_show (track_widget);
               num_children[1]++;
 
               /* Append a separator. The last one will be destroyed later */
-              last_separator[1] = gtk_vseparator_new ();
-              gtk_table_attach (GTK_TABLE (views[1]), last_separator[1], 
-                                num_children[1], num_children[1] + 1, 0, 2, GTK_SHRINK, GTK_FILL|GTK_EXPAND, 0, 0);
+              last_separator[1] = gtk_separator_new (GTK_ORIENTATION_HORIZONTAL);
+              gtk_box_pack_start(GTK_BOX(views[1]), last_separator[1], FALSE, FALSE, 1);
               gtk_widget_show (last_separator[1]);
               num_children[1]++;
 
               /* Add the track to the hash table */
-              g_hash_table_insert (mixer_container->widgets, g_strdup (track_label), track_widget);
+              g_object_set_data (G_OBJECT(track), "track-widget", track_widget);
+              mixer_container->widgets = g_list_append (mixer_container->widgets, track_widget);
               break;
 
             case XFCE_MIXER_TRACK_TYPE_SWITCH:
               track_widget = xfce_mixer_switch_new (mixer_container->card, track);
-              gtk_table_attach (GTK_TABLE (views[2]), track_widget,
-                                0, 1, num_children[2], num_children[2] + 1, GTK_FILL|GTK_EXPAND, GTK_SHRINK, 0, 0);
+              g_object_set (G_OBJECT (track_widget), "halign", GTK_ALIGN_FILL, "hexpand", TRUE, NULL);
+              gtk_grid_attach (GTK_GRID (views[2]), track_widget, 0, num_children[2], 1, 1);
               gtk_widget_show (track_widget);
 
               num_children[2]++;
 
               /* Add the track to the hash table */
-              g_hash_table_insert (mixer_container->widgets, g_strdup (track_label), track_widget);
+              g_object_set_data (G_OBJECT(track), "track-widget", track_widget);
+              mixer_container->widgets = g_list_append (mixer_container->widgets, track_widget);
               break;
 
             case XFCE_MIXER_TRACK_TYPE_OPTIONS:
-              label_alignment = gtk_alignment_new (0, 0, 0, 0);
-              gtk_table_attach (GTK_TABLE (views[3]), label_alignment,
-                                0, 1, num_children[3], num_children[3] + 1, GTK_FILL, GTK_SHRINK, 0, 0);
-              gtk_widget_show (label_alignment);
-
               option_track_label = g_strdup_printf ("%s:", track_label);
               track_label_widget = gtk_label_new (option_track_label);
-              gtk_container_add (GTK_CONTAINER (label_alignment), track_label_widget);
+              g_object_set (G_OBJECT (track_label_widget), "halign", GTK_ALIGN_FILL, NULL);
+              gtk_grid_attach (GTK_GRID (views[3]), track_label_widget, 0, num_children[3], 1, 1);
               gtk_widget_show (track_label_widget);
               g_free (option_track_label);
 
-              option_alignment = gtk_alignment_new (0, 0, 1.0f, 0);
-              gtk_table_attach (GTK_TABLE (views[3]), option_alignment,
-                                1, 2, num_children[3], num_children[3] + 1, GTK_FILL|GTK_EXPAND, GTK_SHRINK, 0, 0);
-              gtk_widget_show (option_alignment);
-
               track_widget = xfce_mixer_option_new (mixer_container->card, track);
-              gtk_container_add (GTK_CONTAINER (option_alignment), track_widget);
+              g_object_set (G_OBJECT (track_widget), "halign", GTK_ALIGN_FILL, "hexpand", TRUE, NULL);
+              gtk_grid_attach (GTK_GRID (views[3]), track_widget, 1, num_children[3], 1, 1);
               gtk_widget_show (track_widget);
 
               num_children[3]++;
 
               /* Add the track to the hash table */
-              g_hash_table_insert (mixer_container->widgets, g_strdup (track_label), track_widget);
+              g_object_set_data (G_OBJECT(track), "track-widget", track_widget);
+              mixer_container->widgets = g_list_append (mixer_container->widgets, track_widget);
               break;
             }
         }
@@ -390,30 +469,24 @@ xfce_mixer_container_create_contents (XfceMixerContainer *mixer_container)
       label1 = gtk_label_new (_("No controls visible"));
       gtk_widget_show (label1);
 
-      alignment = gtk_alignment_new (0.5, 0.5, 0, 0);
-      gtk_widget_show (alignment);
-
-      vbox = gtk_vbox_new (6, FALSE);
-      gtk_container_add (GTK_CONTAINER (alignment), vbox);
+      vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
+      g_object_set (G_OBJECT (vbox), "halign", GTK_ALIGN_CENTER, "hexpand", TRUE, "valign", GTK_ALIGN_CENTER, "vexpand", TRUE,
+                    "border-width", 6, NULL);
       gtk_widget_show (vbox);
 
       label2 = gtk_label_new (NULL);
       gtk_label_set_markup (GTK_LABEL (label2), _("<span weight=\"bold\" size=\"larger\">No controls visible</span>"));
-      gtk_label_set_max_width_chars (GTK_LABEL (label2), 80);
-      gtk_label_set_line_wrap (GTK_LABEL (label2), TRUE);
-      gtk_misc_set_alignment (GTK_MISC (label2), 0.0, 0.5);
+      g_object_set (G_OBJECT (label2), "max-width-chars", 80, "xalign", 0.0, "wrap", TRUE, NULL);
       gtk_box_pack_start (GTK_BOX (vbox), label2, FALSE, TRUE, 0);
       gtk_widget_show (label2);
 
       label3 = gtk_label_new (NULL);
       gtk_label_set_markup (GTK_LABEL (label3), _("In order to toggle the visibility of mixer controls, open the <b>\"Select Controls\"</b> dialog."));
-      gtk_label_set_max_width_chars (GTK_LABEL (label3), 80);
-      gtk_label_set_line_wrap (GTK_LABEL (label3), TRUE);
-      gtk_misc_set_alignment (GTK_MISC (label3), 0.0, 0.5);
+      g_object_set (G_OBJECT (label3), "max-width-chars", 80, "xalign", 0.0, "wrap", TRUE, NULL);
       gtk_box_pack_start (GTK_BOX (vbox), label3, FALSE, TRUE, 0);
       gtk_widget_show (label3);
 
-      gtk_notebook_append_page (GTK_NOTEBOOK (mixer_container), alignment, label1);
+      gtk_notebook_append_page (GTK_NOTEBOOK (mixer_container), vbox, label1);
     }
 
   g_object_unref (preferences);
@@ -428,9 +501,9 @@ xfce_mixer_container_update_contents (XfceMixerContainer *mixer_container)
   gint i;
 
   g_return_if_fail (IS_XFCE_MIXER_CONTAINER (mixer_container));
-  g_return_if_fail (mixer_container->widgets != NULL);
 
-  g_hash_table_remove_all (mixer_container->widgets);
+  g_list_free(mixer_container->widgets);
+  mixer_container->widgets = NULL;
 
   /* Remember active tab */
   current_tab = gtk_notebook_get_current_page (GTK_NOTEBOOK (mixer_container));
@@ -475,7 +548,7 @@ xfce_mixer_container_bus_message (GstBus             *bus,
         gst_mixer_message_parse_mute_toggled (message, &track, &muted);
         label = xfce_mixer_get_track_label (track);
         xfce_mixer_debug ("Track '%s' was %s", label, muted ? "muted" : "unmuted");
-        widget = g_hash_table_lookup (mixer_container->widgets, label);
+        widget = g_object_get_data(G_OBJECT(track), "track-widget");
 
         if (IS_XFCE_MIXER_TRACK (widget))
           xfce_mixer_track_update_mute (XFCE_MIXER_TRACK (widget));
@@ -486,7 +559,7 @@ xfce_mixer_container_bus_message (GstBus             *bus,
         gst_mixer_message_parse_record_toggled (message, &track, &record);
         label = xfce_mixer_get_track_label (track);
         xfce_mixer_debug ("Recording on track '%s' was %s", label, record ? "turned on" : "turned off");
-        widget = g_hash_table_lookup (mixer_container->widgets, label);
+        widget = g_object_get_data(G_OBJECT(track), "track-widget");
 
         if (IS_XFCE_MIXER_TRACK (widget))
           xfce_mixer_track_update_record (XFCE_MIXER_TRACK (widget));
@@ -497,7 +570,7 @@ xfce_mixer_container_bus_message (GstBus             *bus,
         gst_mixer_message_parse_volume_changed (message, &track, &volumes, &num_channels);
         label = xfce_mixer_get_track_label (track);
         xfce_mixer_debug ("Volume on track '%s' changed to %i", label, volumes[0]);
-        widget = g_hash_table_lookup (mixer_container->widgets, label);
+        widget = g_object_get_data(G_OBJECT(track), "track-widget");
 
         if (IS_XFCE_MIXER_TRACK (widget))
           xfce_mixer_track_update_volume (XFCE_MIXER_TRACK (widget));
@@ -506,7 +579,7 @@ xfce_mixer_container_bus_message (GstBus             *bus,
         gst_mixer_message_parse_option_changed (message, &options, &option);
         label = xfce_mixer_get_track_label (GST_MIXER_TRACK (options));
         xfce_mixer_debug ("Option '%s' was set to '%s'", label, option);
-        widget = g_hash_table_lookup (mixer_container->widgets, label);
+        widget = g_object_get_data(G_OBJECT(track), "track-widget");
 
         if (IS_XFCE_MIXER_OPTION (widget))
           xfce_mixer_option_update (XFCE_MIXER_OPTION (widget));
